declare module BA.Ui.Widgets {
    class Widget {
        private _WidgetLoadState;
        private _DefaultFocusOnNextLoaded;
        private _DefaultFocusCallbackOnNextLoaded;
        private _TriggerOnContentLoadedAfterNextLoad;
        protected _SetProvidedParameterDelay: number;
        protected IconDefaultUrlSuffix: string;
        protected Collapsible: boolean;
        Id: BA.Classes.Guid;
        WidgetInstanceOf: BA.Classes.Guid;
        ParentWidgetId: BA.Classes.Guid;
        PageId: BA.Classes.Guid;
        PageConfigId: BA.Classes.Guid;
        DelayedLoad: boolean;
        ProvidedParameters: string;
        IsAutoCollapsed: boolean;
        IsCollapsed: boolean;
        WidgetType: string;
        HandleParameterChangedNeeded: boolean;
        Previewable: boolean;
        PreviewMode: boolean;
        static ResizeObserver: ResizeObserver;
        private _SetProvidedParameterTimeoutHandler;
        private _LatestParameter;
        private _LatestParameterListJsonString;
        private _additionalJQEventHandlers;
        private _isExpandingOrCollapsing;
        /**
         * Creates the instance of the widget and initializes it.
         *
         * @param widgetId Id of the widget in the page configuration.
         * @param pageConfigId Id of the page configuration.
         * @param widgetType Type of widget being initialized.
         * @param delayedLoad If the loading of the content should be deferred. Default false.
         * @param inWidgetCatalogue If the widget is being rendered in the catalogue.
         */
        constructor(widgetId: BA.Classes.Guid, pageConfigId: BA.Classes.Guid, widgetType: string, delayedLoad?: boolean, previewable?: boolean, previewMode?: boolean, inWidgetCatalogue?: boolean);
        CreateNewWidgetInstance(newId?: BA.Classes.Guid): Widget;
        IsInCatalogue(): boolean;
        InitialLoad(): void;
        /**
        * Loads the content of the widget from the server.
        *
        * @param blockWidget If the widget should be blocked while loading. Default true.
        * @param async If the load of content should work asynchrounously. Default true.
        * @param forceCollapseAction If the collapse should be forced
        */
        protected LoadWidgetContent(blockWidget?: boolean, async?: boolean, forceCollapseAction?: boolean): void;
        /**
         * Loads the content of the widget from the server.
         *
         * @param blockWidget If the widget should be blocked while loading. Default true.
         * @param async If the load of content should work asynchrounously. Default true.
         * @param forceCollapseAction If the collapse should be forced
         */
        protected LoadWidgetPreviewContent(blockWidget?: boolean, async?: boolean, forceCollapseAction?: boolean): void;
        /**
         * Getter for the widget load state property
         */
        get WidgetLoadState(): WidgetState;
        /**
         * Sets the widget load state and carries out the corresponding actions needed depending on the load state set.
         *
         * @param value New load state.
         * @param stateMessage If the state should include a message (only by blocking states). Default empty.
         * @param forceCollapseAction If the collapse should be forced
         */
        protected SetWidgetState(value: WidgetState, stateMessage?: string, forceCollapseAction?: boolean): void;
        /**
         * Helper function to show an error message on the widget and display it on the error state.
         *
         * @param errorText Message to show. Default empty.
         */
        protected ShowErrorMessage(errorText?: string): void;
        /**
         * Cleans the ui deleting the classes or attributes that are added when setting a state
         */
        protected RemoveWidgetStateAndTexts(): void;
        /**
         * Helper function to block the widget showing also a message on it. The widget will be displayed on the blocked state.
         *
         * @param blockedText Message to show. Default empty.
         */
        protected BlockWidgetUI(blockedText?: string): void;
        /**
         * Unblocks the widget and deletes the text shown, if any
         */
        protected UnblockWidgetUI(): void;
        /**
         * Info about the parameter that the widget uses, if any.
         */
        get ParameterInfo(): any;
        /**
         * Triggers the parameter change event from the source of the parameter (page or other widget). This is used for example when the widget changes that parameter and other widgets should be reloaded with the new data.
         */
        protected TriggerSourceParameterChangedEvent(): void;
        /**
         * Triggers the parameter change event from the widget.
         */
        protected TriggerParameterChangedEvent(parameterName: string, changedBy?: BA.Classes.Guid): void;
        /**
         * Attaches the widget to the parameter changes events depending on the parameter settings
         */
        protected AttachToParameterSourceEvents(): void;
        /**
         * Returns a json string of the parameter from the parameter source.
         */
        ReadSourceParameterJsonString(): string;
        /**
         * Returns a json string of a list containing the parameter provided by the widget. If the widget is multi instance, the list will contain the parameters of all instances
         *
         * @param parameterName Name of the parameter type to get.
         */
        GetAllProvidedParametersJsonString(parameterName: string): string;
        /**
         * Returns the parameter provided by the widget.
         *
         * @param parameterName Name of the parameter to get.
         */
        protected GetProvidedParameter<T extends Parameters.WidgetParameterBase>(parameterName: string): T;
        /**
         * Saves a parameter on the widget for other widgets to read it.
         *
         * @param parameterName Name of the parameter to set.
         * @param value Widget parameter to save.
         * @param onlyIfDifferent Optional, default: true. If set to true it will only change the provided parameter and trigger the ui event if value is different to the current provided parameter.
         */
        protected SetProvidedParameter<T extends Parameters.WidgetParameterBase>(parameterName: string, value: T, onlyIfDifferent?: boolean): void;
        private SetAllProvidedParametersToNull;
        /**
         * Default handling for the parameter change event.
         *
         * @param blockWidget If the widget should be blocked during the handling. Default true.
         * @param async If it should be done asynchronously. Default true.
         * @param forceCollapseAction If the collapse should be forced
         * @param newParameter New parameter from the source triggering the event
         * @param parametersListJsonString Json string of list of parameters from the source. If the source is a multi instance widget, this list contains the parameters of all the instances. Otherwise it is a list of parameters containing only newParameter
         */
        protected OnParameterChanged(blockWidget?: boolean, async?: boolean, forceCollapseAction?: boolean, newParameter?: Parameters.WidgetParameterBase, parametersListJsonString?: string): void;
        /**
         * Returns the JQuery object corresponding this widget.
         */
        GetWidgetJQ(): JQuery;
        /**
         * Returns the JQuery object corresponding this widget's content.
         */
        protected GetWidgetContentJQ(): JQuery;
        /**
         * Returns the JQuery object corresponding this widget's preview content
         */
        protected GetWidgetPreviewContentJQ(): JQuery;
        /**
         * Returns a string containing the id selector for JQuery.
         */
        GetWidgetJQIdSelector(): string;
        SetParentWidgetId(newParentId: BA.Classes.Guid): void;
        /**
         * Adds an action to the widgets actions, if the actionName is not yet added.
         *
         * @param iconName Icon to be shown in the action button.
         * @param actionName Identifier of the action of the widget.
         * @param tooltip Text to be shown as tooltip.
         * @param sortOrder Desired position of the action.
         * @param onClick Function to be called when clicking the action.
         * @param triggerUIEvent If the WidgetActinClicked UI Event should be triggered
         */
        AddWidgetAction(iconName: string, actionName: string, tooltip: string, sortOrder: number, onClick: Function, triggerUIEvent?: boolean): void;
        /**
         * Find out if a an action is added to the widget.
         *
         * @param actionName Name of the widget action to check for.
         */
        GetWidgetActionIsAvailable(actionName: string): boolean;
        /**
         * Centralized function to check which actions should be added to the widget and call the corresponding functions to add them.
         */
        protected AddActions(): void;
        /**
         * Adds the action for removing a widget to the widget's actions,
         * if the widget can be removed to the widget catalogue.
         */
        AddRemoveAction(): void;
        /**
         * Adds the actions for toggling between preview and complete widget to the widget's actions.
         */
        protected AddPreviewActions(): void;
        /**
         * Adds the action for refreshing a widget to the widget's actions.
         */
        protected AddWidgetSettingsDialogAction(): void;
        /**
         * Adds the action for refreshing a widget to the widget's actions.
         */
        protected AddRefreshAction(): void;
        /**
         * Adds the action for collapsing a widget to the widget's actions.
         */
        protected AddCollapseAction(): void;
        /**
         * Adds the action for maximizing a widget to the widget's actions.
         */
        protected AddMaximizeAction(): void;
        /**
         * Action method: refresh a widget.
         */
        Refresh(blockWidget?: boolean, async?: boolean): void;
        /**
         * Action method: maximize and restore a widget.
         */
        ToggleMaximize(): void;
        /**
         * Helper function to add a title if necessary
         * */
        private AddOrRefreshTitle;
        /**
         * Helper function to add a title if necessary and bind click event to it
         * */
        private AddOrRefreshTitleAndClickEvent;
        /**
         * Changes the widget between preview and complete widget
         */
        SetPreviewMode(on: boolean): void;
        /**
         * Action method: objectcollapse and expand a widget.
         *
         * @param setWidgetSetting Set user spefic widget setting for collapsed status, default = true.
         */
        ToggleCollapse(setWidgetSetting?: boolean): void;
        private Expand;
        private Collapse;
        /**
         * Internal DefaultFocus handling for when the widget is not loaded.
         *
         * @param callback Function to be called after executing the DefaultFocus.
         */
        DefaultFocusInternal(callback?: Function): void;
        /**
         * Standard DefaultFocus handler.
         *
         * @param callback Function to be called after executing the DefaultFocus.
         */
        DefaultFocus(callback?: Function): void;
        /**
         * Hides the widget's actions container.
         *
         * @param immediate If it has to happen immediately
         * @param $actionsContainer The actions container, if it is already known. If null, the widget's actions container will be retrieved and hidden.
         */
        HideWidgetActions(immediate?: boolean, $actionsContainer?: JQuery): void;
        /**
         * Handler for when the widget is no longer hovered. This sets the correct visibility of the actions.
         *
         * @param $actionsContainer
         * @param $actionItem
         */
        private HandleLostHover;
        /**
         * Get a user specific widget setting.
         *
         * @param option The name of the setting as string.
         * @param scope Enum value of scope.
         * @param onDone Function to be called when done for custom handling.
         * @param onError Function to be called if an error occurs for custom error handling.
         */
        protected GetWidgetSetting(option: string, onDone?: (response: any) => void, onError?: (response: any) => void): any;
        /**
        * Get several user specific widget settings as anonymous object (the requested options will be properties of the returned object).
        *
        * @param options The names of the settings as string.
        * @param scope Enum value of scope.
        * @param onDone Function to be called when done for custom handling.
        * @param onError Function to be called if an error occurs for custom error handling.
        */
        protected GetWidgetSettings(options: string[], onDone?: (response: any) => void, onError?: (response: any) => void): any;
        /**
         * Get several user specific widget settings as anonymous object (the requested options will be properties of the returned object).
         * If the widget settings do not exist for the identified widget, return the config values instead.
         *
         * @param options The names of the settings as string.
         * @param scope Enum value of scope.
         * @param onDone Function to be called when done for custom handling.
         * @param onError Function to be called if an error occurs for custom error handling.
         */
        protected GetWidgetSettingsOrConfigValues(options: string[], onDone?: (response: any) => void, onError?: (response: any) => void): any;
        /**
         * Remove a user specific widget setting.
         *
         * @param option The name of the setting as string.
         * @param scope Enum value of scope.
         * @param onDone Function to be called when done for custom handling.
         * @param onError Function to be called if an error occurs for custom error handling.
         */
        protected RemoveWidgetSetting(option: string, onDone?: (response: any) => void, onError?: (response: any) => void): any;
        /**
         * Set a user specific widget setting.
         *
         * @param option The name of the setting as string.
         * @param scope Enum value of scope.
         * @param value Value of the UI setting.
         * @param onDone Function to be called when done for custom handling.
         * @param onError Function to be called if an error occurs for custom error handling.
         */
        protected SetWidgetSetting(option: string, value: any, onDone?: (response: any) => void, onError?: (response: any) => void): void;
        /**
         * Function to be called when widget settings have been changed
         */
        protected OnWidgetSettingsChanged(isSuccess: boolean, changesJson: string, triggerContentReload?: boolean): void;
        /**
         * Function that is called when the widget settings dialog has been closed
         */
        OnWidgetSettingsDialogClosed(isSuccess: boolean, changesJson: string, triggerContentReload?: boolean): void;
        BeforeRemoving(): boolean;
        OnRemovedInternal(): void;
        OnRemoved(): void;
        /**
         * Function called when a widget is unloaded, for example after removing it, to remove also any dependencies, UI event handler or other necessary steps
         */
        private UnloadInternal;
        Unload(tabClosing?: boolean): void;
        private DetachParameterEvents;
        IsMultiInstance(): boolean;
        IsInstanceWidget(): boolean;
        /**
         * Function called when the instance of the widget is created, regardless of wether the content is loaded or not.
         */
        Init(): void;
        /**
         * Function to attach events that will only be triggered when the widget is in a drag and drop area widget.
         */
        AttachDragAndDropEvents(): void;
        /**
         * Function called after the widget's content is loaded.
         */
        OnContentLoaded(): void;
        /**
         * Function called after the widget's preview content is loaded.
         */
        OnPreviewContentLoaded(): void;
        /**
         * Function called before the widget collapses. Return false to prevent the widget from collapsing.
         */
        protected BeforeCollapsing(): boolean;
        /**
         * Function called after the widget collapses.
         */
        protected OnCollapsed(): void;
        /**
         * Function called before the widget expands. Return false to prevent the widget from expanding.
         */
        protected BeforeExpanding(): boolean;
        /**
         * Function called after the widget expands.
         */
        protected OnExpanded(): void;
        /**
        * Function called before the widget maximizes. Return false to prevent the widget from maximizing.
        */
        protected BeforeMaximizing(): boolean;
        /**
        * Function called before the widget restores its size. Return false to prevent the widget from restoring its size.
        */
        protected BeforeSizeRestoring(): boolean;
        /**
         * Function called after the widget is maximized.
         */
        protected OnMaximized(): void;
        /**
         * Function called after the widget's size is restored.
         */
        protected OnSizeRestored(): void;
        /**
         * Function called when a widget changes the parameter provided by this widget
         *
         * @param parameterName The name of the parameter changed
         * @param changedBy The Id of the widget that changed the parameter
         */
        protected OnParameterExternallyChanged(parameterName: string, changedBy: BA.Classes.Guid): void;
        /**
         * Function called after the parameter source widget is removed.
         *
         * @param removedWidget the widget object being removed.
         */
        protected OnParameterSourceRemoved(data: CustomData): void;
        /**
         * Function called by the window management when the browser window is resized.
         * Should implement stuff to do when this happens, practiaclly moved from 'var onResize = function...' from some
         * of the cshtml files that implement widgets.
         * To be overridden in child classes where needed.
         *
         * @param evt the resize-event, might be null because this gets called without event in a couple of places
         */
        OnResize(evt?: Event): void;
        /**
         * Function called by the tab system when i.e. the grid views on that tab need to be refreshed.
         * Moved from 'var onShow = function...' from some of the cshtml files that implement widgets.
         * To be overridden in child classes where needed.
         */
        OnShow(): void;
    }
    /** Constants for standard event types */
    class WidgetEventTypes {
        /** Type of event triggered when widgets change their provided parameter */
        static readonly WidgetParameterUpdated: string;
        /** Type of event triggered when widgets action is clicked */
        static readonly WidgetActionClicked: string;
        /** Type of event triggered when drag starts */
        static readonly WidgetDragStart: string;
        /** Type of event triggered when a widget is dragged */
        static readonly WidgetDragged: string;
        /** Type of event triggered when widgets drop */
        static readonly WidgetDropped: string;
        /** Type of event triggered when widgets drop in a container */
        static readonly WidgetDroppedIn: string;
        /** Type of event triggered when widgets drop out of a container */
        static readonly WidgetDroppedOut: string;
        /** Type of event triggered when dragged widgets hovers over a container */
        static readonly WidgetHoverIn: string;
        /** Type of event triggered when dragged widgets no longer hovers over a container */
        static readonly WidgetHoverOut: string;
        /** Type of event triggered when widgets drag ends */
        static readonly WidgetDragEnd: string;
        /** Type of event triggered when widgets drag gets cancelled */
        static readonly WidgetDragCancelled: string;
        /** Type of event triggered when a widget is removed */
        static readonly WidgetRemoved: string;
    }
}
